//  リスト12.7 Timerを使ったアニメーション
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

class TimerAnimation extends JPanel implements ActionListener {
    int number = -1;
    int cell = 20;
    int interval;
    Image[] image;
    MediaTracker tracker;
    Timer timer;

    TimerAnimation() {
        //イメージのロード
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        tracker = new MediaTracker(this);
        image = new Image[cell];
        for (int i = 1; i <= cell; i++) {
           image[i-1] = 
              toolkit.getImage("rabbit/rabbit" + i + ".gif");
           tracker.addImage(image[i-1], 0);
        }
        
        interval = 150;                    //描画の間隔(ミリ秒)
        timer = new Timer(interval, this); //タイマーの生成
        timer.setInitialDelay(0);          //すぐに始める
        timer.setCoalesce(true);           //イベントをまとめる
     
        setBackground(Color.white);       //パネルの背景色
        try{         
           tracker.waitForAll();          //ロードを開始
	    }catch(InterruptedException e){ return; }
        startAnimation();                 //アニメーションの開始
    }

    public synchronized void startAnimation() {
       if (!timer.isRunning()) {
               timer.start();       //アニメーションの開始
       }
    }

    public synchronized void stopAnimation() {
        if (timer.isRunning()) {
            timer.stop();          //アニメーションを止める
        }
    }

    public void actionPerformed(ActionEvent e) { //アクションイベント処理
        number++;  //次に表示するイメージの通し番号
        repaint(); //描画
    }

    public void paintComponent(Graphics g) {
       super.paintComponent(g); //背景の描画
       int x = getWidth()/2-30; //ウィンドウのほぼ中央にイメージを表示
       int y = getHeight()/2-30;
       try {
           g.drawImage(image[number%cell], x, y, this);
       } catch (ArrayIndexOutOfBoundsException e) { return; }
                        //numberが-1の時に描画が呼ばれた時の例外処理
    }

    public static void main(String[] args) {
        JFrame f = new JFrame("Timer Animation");
        TimerAnimation anime = new TimerAnimation();
        f.getContentPane().add(anime, BorderLayout.CENTER);
        f.setSize(new Dimension(100, 100));
        f.setVisible(true);
    }
}





