//リスト 11.4 アニメーション
import java.awt.*;
import javax.swing.*;

public class SimpleAnimation extends JPanel implements Runnable {
    int number = -1;
    int cell = 20;
    int interval;
    Thread animatorThread;
    Image[] image;
    MediaTracker tracker;

    public SimpleAnimation() {
        //イメージのロード
        tracker = new MediaTracker(this);
        Toolkit toolkit = Toolkit.getDefaultToolkit();
        image = new Image[cell];
        for (int i = 1; i <= cell; i++) {
            image[i-1] = 
               toolkit.getImage("rabbit/rabbit" + i + ".gif");
            tracker.addImage(image[i-1], 0);
        }

        interval = 150;                  //描画の間隔(ミリ秒)
        setBackground(Color.white);      //パネルの背景色
        startAnimation();                //アニメーションの開始
    }

    public synchronized void startAnimation() {
       //アニメーションを開始
       if (animatorThread == null) {
           animatorThread = new Thread(this);
        }
        animatorThread.start();
    }

    public synchronized void stopAnimation() {
        if (animatorThread != null) {//アニメーションスレッドを止める
            animatorThread = null;
        }
    }

    public void run() {
      try{
         tracker.waitForAll();
	  }catch(InterruptedException e){ return; }
      while(animatorThread == Thread.currentThread()){
        number++;  //次に表示するイメージの通し番号
        repaint(); //描画
        try{
          Thread.sleep(interval);
        }catch(InterruptedException e){ break; }
      }
    }

    public void paintComponent(Graphics g) {
       super.paintComponent(g); //背景の描画
       int x = getWidth()/2-30;  //ウィンドウのほぼ中央にイメージを表示
       int y = getHeight()/2-30;
       try {
           g.drawImage(image[number%cell], x, y, this);
       } catch (ArrayIndexOutOfBoundsException e) { return; }
                                  //numberが-1の時に描画が呼ばれた時の例外処理
    }

    public static void main(String[] args) {
        JFrame f = new JFrame("Simple Animation");
        SimpleAnimation anime = new SimpleAnimation();
        f.getContentPane().add(anime, BorderLayout.CENTER);
        f.setSize(new Dimension(100, 100));
        f.setVisible(true);
    }
}





