import java.awt.Color;
/** 任意の点に色をつけられるTurtleFrameの拡張 */
public class ColorFrame extends TurtleFrame {
  private Turtle drawer;				//作図のためのタートル

  /**大きさwidth x height の点画用フレームを作る*/
  public ColorFrame(int width, int height) {
    super(width,height);
    drawer = new Turtle(0,0,0);
    add(drawer);
    drawer.withKame = false;
  }
  /** 引数無しの場合は大きさ400x400の点画用フレームを作る*/
  public ColorFrame(){ this(400,400); }
	
  /** 座標(x,y)にcolorNumberに対応した色をつける。
    * @param x   座標のx成分
    * @param y	 座標のy成分
    * @param colorNumber 色番号(負の場合は黒)
    */
  public void plot(int x, int y, int colorNumber) {
    drawer.setColor(getColor(colorNumber)); 	//線の色を変える

    //タートルを(x,y)へ移動させることで点を打つ。
		
    //現在のタートルの位置と(x,y)が離れている場合、線が引かれてしまわ
    //ないように、一度ペンを上げて(x,y)へ移動し、ペンを降ろしてからも
    //う一度(x,y)へ移動する。 ただし、up/downは遅いので、タートルが
    //(x,y)に隣接する位置にいる場合には、直接(x,y)へ移動する

    int dx = drawer.getX()-x, dy = drawer.getY()-y; //目的の位置との差が
    if (Math.abs(dx)+Math.abs(dy) != 1) {	    //1より大きい場合には
      drawer.up();				    //ペンを上げて
      drawer.moveTo(x,y,0);			    //目的の位置に移動して
      drawer.down();				    //ペンを下ろしておく
    }
    drawer.moveTo(x,y,0);			    //(x,y)に色をつける
  }
	
  /** 整数kを適当な色に変換する */
  public static Color getColor(int k) {
    if (k < 0) {
      return Color.black;		//負の場合は黒
    } else {
      float hue        = k/30.0F;	// 色相
      float saturation = 1;		// 彩度
      float brightness = 1;		// 明度
      // 色相・彩度・明度からRGB値を計算
      int rgb = Color.HSBtoRGB(hue,saturation,brightness);
      // RGB値から Color オブジェクトを作って返す
      return new Color(rgb);	
    }
  }

}
