// 塗りつぶしをするフレームとタートル
import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.util.Vector;

import javax.swing.JComponent;

// フレームを拡張する
class FillingFrame extends TurtleFrame {

	// フレームに表示する1つの多角形 (cf. TurtlePanel.Line)
	class FilledPolygon extends java.awt.Polygon {
		public Color c;
		public FilledPolygon(int[] xs, int[] ys, Color c) {
			super(xs, ys, xs.length);
			this.c = c;
		}
	}
	// 多角形を1つ追加する
	public void addPolygonElement(int[] xs, int[] ys, Color c) {
		history.add(new FilledPolygon(xs, ys, c));
	}

	FillingFrame(int width, int height) {
		super(width, height);
		
		// TurtlePanelが追加した部品を置き換える
		Component[] cs = getComponents();
		for (int i = 0; i < cs.length; i++)
			remove(cs[i]);
		
		add(new JComponent() { // Anonymous class
			public void paint(Graphics g) {
				//途中までは元と同じ
				synchronized (mappedLock) {
					if (!mapped) {
						mapped = true;
						mappedLock.notify();
					}
				}
				if (mesh) {
					drawmesh(g);
				}
				for (int i = 0; i < history.size(); i++) {
					// Line以外も描画できるように拡張
					if (history.elementAt(i) instanceof Line) {
						Line line = (Line) history.elementAt(i);
						g.setColor(line.c);
						g.drawLine(line.bx, line.by, line.ex, line.ey);
					} else if (history.elementAt(i) instanceof FilledPolygon) {
						//多角形の表示
						FilledPolygon polygon =
							(FilledPolygon) history.elementAt(i);
						g.setColor(polygon.c);
						g.fillPolygon(polygon);
					}
				}
				//以下は元と同じ
				for (int i = 0; i < turtles.size(); i++) {
					Turtle t = (Turtle) turtles.elementAt(i);
					t.show(g);
				}
			}
		});
		frame.pack(); // 置き換えた部品を見せる
	}
	FillingFrame() {
		this(400, 400);
	}
}

//多角形を塗りつぶせるタートル
class FillingTurtle extends Turtle {
	Vector points = new Vector();
	boolean recording = false;

	public void startPolygon() {
		points.clear();
		recording = true;
		recordCurrentPosition();
	}
	public void fillPolygon() {
		int[] xs = new int[points.size()];
		int[] ys = new int[points.size()];
		for (int i = 0; i < xs.length; i++) {
			xs[i] = ((java.awt.Point) points.get(i)).x;
			ys[i] = ((java.awt.Point) points.get(i)).y;
		}
		((FillingFrame) f).addPolygonElement(xs, ys, c);
		kameShow(moveWait);
	}
	private void recordCurrentPosition() {
		if (recording)
			points.addElement(new java.awt.Point(getX(), getY()));
	}
	public void fd(int n) {
		super.fd(n);
		recordCurrentPosition();
	}
	public int moveTo(double x, double y, double aa) {
		int r = super.moveTo(x, y, aa);
		recordCurrentPosition();
		return r;
	}
}

// 使用例
public class Filling {

	public static void main(String[] args) {
		FillingFrame f = new FillingFrame();
		FillingTurtle t = new FillingTurtle();
		f.add(t);
		t.up();
		t.startPolygon();			//多角形を描き始めると宣言
		for (int i = 0; i < 5; i++) {
			t.fd(100);				//移動した点は多角形の頂点
			t.rt(360 / 5);
		}
		t.setColor(Color.pink);
		t.fillPolygon();			//多角形を塗りつぶす
	}
}
