//
// Turtle Graphics Library extension for teaching OO concepts
// as a modification of Tatsuya Hagino's Turtle Graphics Library
// Copyright (C) 2000, Hideki Tsuiki, Kyoto University

// Turtle Graphics Library for Information Processing I
// Copyright (C) 1998, Tatsuya Hagino, Keio University
//
// Permission to use, copy, modify, and distribute this software
// for educational purpose only is hereby granted, provided that
// the above copyright notice appear in all copies and that both
// the copyright notice and this permission notice appear in
// supporting documentation.  This software is provided "as is"
// with no warranty.


import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import java.util.Vector;


/** タートルを表示する枠のクラス。 */

public class TurtleFrame extends TurtlePanel implements ActionListener, ItemListener { 
  JFrame frame;
  // menu
  JRadioButtonMenuItem[] speedMenu;
  static String[] speedMenuString = {"no kame", "fast", "normal", "slow"};

  /** TurtlePanel を、width x height の大きさで作成する。 */
  public TurtleFrame(int width, int height) {
    super(width, height);
    frame = new JFrame("Turtle");
    JMenuBar menubar = new JMenuBar();
    JMenu file = new JMenu("File", true);
    menubar.add(file);
    JMenuItem quit = new JMenuItem("Quit");
    quit.addActionListener(this);
    quit.setActionCommand("quit");
    file.add(quit);

    JMenu speed = new JMenu("Speed", true);
    menubar.add(speed);
    speedMenu = new JRadioButtonMenuItem[speedMenuString.length];
    ButtonGroup bg = new ButtonGroup();
    for (int i = 0; i < speedMenuString.length; i++) {
      speedMenu[i] = new JRadioButtonMenuItem(speedMenuString[i], false);
      speedMenu[i].addItemListener(this);
      speed.add(speedMenu[i]);
      bg.add(speedMenu[i]);
    }
    speedMenu[Turtle.speedNormal].doClick();
    frame.setJMenuBar(menubar);
    Container c = frame.getContentPane();
    c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
    setAlignmentX(JComponent.CENTER_ALIGNMENT);
    c.add(this);
    frame.pack();

    frame.setDefaultCloseOperation(3);  // EXIT_ON_CLOSE
    
    frame.setVisible(true);
    synchronized(mappedLock) {
      if (!mapped) {
	try {
	  mappedLock.wait();
	}
	catch (InterruptedException e) {
	}
      }
    }
    
  }

  /** TurtleFrame を、デフォルトの大きさ(400x400) で作成する。 */
  public TurtleFrame() {
    this(400,400);
  }

  // exit
  public void exit() {
    System.exit(0);
  }

  // menu bar event handler
  public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd.equals("quit")) System.exit(0);
    else if (cmd.equals("restart")) /*restart()*/;
    else if (cmd.equals("ps")) /*savePostscript()*/; 
  }

  // menu event handler
  public void itemStateChanged(ItemEvent e) {
    JRadioButtonMenuItem s = (JRadioButtonMenuItem)e.getItem();
    for (int i = 0; i < speedMenuString.length; i++) {
      if (s == speedMenu[i]) {
	Turtle.speedAll(i);
	break;
      }
    }
  }
  
  // the following definitions until end of the TurtleFrame class are 
  // added by Hidehiko Masuhara October 2004.
  // they will enable to open dialogs to get numbers or strings
  // from the users.  The dialog suspends the execution of the caller
  // thread, which is easier to the beginners.
  
  static class WaitingDialog extends JDialog implements ActionListener {
    private boolean waiting = true;
    private JTextField field;
    public synchronized void actionPerformed(ActionEvent e) {
        waiting = false;
        notifyAll();
    }
    public synchronized String getString() {
        waiting = true;
        field.selectAll();
        while (waiting) {
            try {
                wait();
            } catch (InterruptedException e) {
                ;
            }
        }
        return field.getText();
    }

    public WaitingDialog(String message) {
        Container cp =getContentPane(); 
        cp.setLayout(new FlowLayout());

        field = new JTextField(10);
        JButton button = new JButton("OK");

        field.addActionListener(this);
        button.addActionListener(this);

        cp.add(new JLabel(message));
        cp.add(field);
        cp.add(button);
        
        this.pack();
    }
  }
 
  /** 整数を入力する枠を表示し、入力が完了したら消去しその数を返す */
  static public int askInt(String message) {
      WaitingDialog d = new WaitingDialog(message);
      d.setVisible(true);
      while (true) {
          try {
              int v = Integer.parseInt(d.getString());
              d.setVisible(false);
              return v;
          } catch (NumberFormatException e) {
              ;
          }
      }
  }
  
  /** 倍精度実数を入力する枠を表示し、入力が完了したら消去しその数を返す */
  static public double askDouble(String message) {
      WaitingDialog d = new WaitingDialog(message);
      d.setVisible(true);
      while (true) {
          try {
              double v = Double.parseDouble(d.getString());
              d.setVisible(false);
              return v;
          } catch (NumberFormatException e) {
              ;
          }
      }
  }
  
  /** 文字列を入力する枠を表示し、入力が完了したら消去しその文字列を返す */
  static public String askString(String message) {
      WaitingDialog d = new WaitingDialog(message);
      d.setVisible(true);
      while (true) {
          String v = d.getString();
          d.setVisible(false);
          return v;
      }
  }
  
}